<?
/**************
	File Name: rss_parser.php

	Author: Herman Veluwenkamp
	Description: Parses RSS into an array.

	Example.

	$rss = new rss_parser();
	$rss->file = 'http://freebsddiary.org/news.php3';
	$rss->parse() or die($rss->error);
	if ($rss->error) print $rss->error;
	print_r($rss->channel);

	2003.12.08
	Edited by woogi (likedy@nate.com)
	2004.02.02
	Edited by woogi (likedy@nate.com)

****************/

    // url_fopen 
    if (ini_get("allow_url_fopen") == 0) {
        ini_set("allow_url_fopen", 1);
    }

// define hooks to rss_parser class as xml functions do not allow object methods as handlers.
function rss_start_element($parser, $name, $attributes) {
  global $rss;
  $rss->start_element($parser, $name, $attributes);
}

function rss_end_element($parser, $name) {
  global $rss;
  $rss->end_element($parser, $name);
}

function rss_character_data($parser, $data) {
  global $rss;
  $rss->character_data($parser, $data);
}


class rss_parser {

// constructor. setup parser options and handlers.
function rss_parser() {
  $this->error = '';
  $this->file = '';

  $this->channel = array();
  $this->data = '';
  $this->stack = array();
  $this->num_items = 0;

  $this->xml_parser = xml_parser_create();
  xml_set_element_handler($this->xml_parser, "rss_start_element", "rss_end_element");
  xml_set_character_data_handler($this->xml_parser, "rss_character_data");
}

function character_data($parser, $data) {
  if (empty($this->data)) $this->data = $data; // concatenate non-parsed data...
  else $this->data .= $data;               // and get rid of white space.
}

function start_element($parser, $name, $attrs) {
  $name = str_replace(":", "_", $name);
  switch($name) {
    case 'RSS':
      break;

    case 'CHANNEL':
      break;

    case 'IMAGE':
      array_push($this->stack, $name);
      break;

    case 'ITEM':
      array_push($this->stack, $name);
      array_push($this->stack, $this->num_items); // push item index.
      $this->item[$this->num_items] = array();
      $this->num_items++;
      break;

    case 'TEXTINPUT':
      array_push($this->stack, $name);
      break;

    default:
      array_push($this->stack, $name);
      break;

  }
}

function end_element($parser, $name) {
  $name = str_replace(":", "_", $name);
  switch ($name) {
    case 'RSS':
      break;

    case 'CHANNEL':
      break;

    case 'IMAGE':
      array_pop($this->stack);
      break;

    case 'ITEM':
      array_pop($this->stack);
      array_pop($this->stack);
      break;

    case 'TEXTINPUT':
      array_pop($this->stack);
      break;

    default: // child element.
      $element = (implode("']['",$this->stack));
      eval("\$this->channel['$element']=\$this->data;"); // this does all the hard work.
      array_pop($this->stack);
      $this->data = '';
      break;
  }
}

function parse() {
	$url_stuff = parse_url(trim($this->file));
	if(!$url_stuff[port]) $url_stuff[port] = 80;

	//  õ!
	$fp = @fsockopen($url_stuff[host], $url_stuff[port], $errno, $errstr, 30);

	if (!$fp)
	{
		$this->error = "Could not open RSS source \"$this->file\".";
    	return false;
	}
	else
	{
		// HTTP ݷ GETõ!
		fputs ($fp, "GET ".$url_stuff[path]."?".$url_stuff['query']." HTTP/1.1\r\n");
		fputs ($fp, "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*\r\n");
		fputs ($fp, "Accept-Language: ko\r\n");
		fputs ($fp, "Accept-Encoding: gzip, deflate\r\n");
		fputs ($fp, "User-Agent: Mozilla/4.0\r\n");
		fputs ($fp, "Host: ".$url_stuff[host]."\r\n");
		fputs ($fp, "Connection: close\r\n");
		fputs ($fp, "Cache-Control: no-cache\r\n");
		fputs ($fp, "\r\n\r\n");

		// HTTP  κ ǳʶڴ.
		while ($data = fgets($fp, 1024))
		{
			if (strstr($data,"<?xml"))
			{
				if (!xml_parse($this->xml_parser, $data, feof($fp))) {
      				$this->error = sprintf("XML error: %s at line %d in %s.",
        			xml_error_string(xml_get_error_code($this->xml_parser)),
        			xml_get_current_line_number($this->xml_parser),
        			$this->file);
      				return false;
      			}
				break;
			}
		}

		while ($data = fgets($fp, 4096))
		{
			if (!xml_parse($this->xml_parser, $data, feof($fp))) {
      			$this->error = sprintf("XML error: %s at line %d in %s.",
        		xml_error_string(xml_get_error_code($this->xml_parser)),
        		xml_get_current_line_number($this->xml_parser),
        		$this->file);
      			return false;
    		}

			//Ѵ⸦ ϱ  .
			if (strstr(strtoupper($data), "</RSS>"))
				break;
		}
		xml_parser_free($xml_parser);
		fclose ($fp);
	}
  return true;
}

} // class rss_parser.
?>